@@ -16,7 +16,7 @@ pub(crate) use esm_resolver::NodeEsmResolver;
 // each release, a better mechanism is preferable, but it's a quick and dirty
 // solution to avoid printing `X-Deno-Warning` headers when the compat layer is
 // downloaded
-static STD_URL_STR: &str = "https://deno.land/std@0.121.0/";
+static STD_URL_STR: &str = "https://deno.land/std@0.122.0/";
 
 static SUPPORTED_MODULES: &[&str] = &[
   "assert",
@@ -102,15 +102,17 @@ fn try_resolve_builtin_module(specifier: &str) -> Option<Url> {
 
 pub(crate) fn load_cjs_module(
   js_runtime: &mut JsRuntime,
-  main_module: &str,
+  module: &str,
+  main: bool,
 ) -> Result<(), AnyError> {
   let source_code = &format!(
-    r#"(async function loadCjsModule(main) {{
-      const Module = await import("{}");
-      Module.default._load(main, null, true);
-    }})('{}');"#,
-    MODULE_URL_STR.as_str(),
-    escape_for_single_quote_string(main_module),
+    r#"(async function loadCjsModule(module) {{
+      const Module = await import("{module_loader}");
+      Module.default._load(module, null, {main});
+    }})('{module}');"#,
+    module_loader = MODULE_URL_STR.as_str(),
+    main = main,
+    module = escape_for_single_quote_string(module),
   );
 
   js_runtime.execute_script(&located_script_name!(), source_code)?;
@@ -138,3 +140,18 @@ pub(crate) fn add_global_require(
 fn escape_for_single_quote_string(text: &str) -> String {
   text.replace(r"\", r"\\").replace("'", r"\'")
 }
+
+pub fn setup_builtin_modules(
+  js_runtime: &mut JsRuntime,
+) -> Result<(), AnyError> {
+  let mut script = String::new();
+  for module in SUPPORTED_MODULES {
+    // skipping the modules that contains '/' as they are not available in NodeJS repl as well
+    if !module.contains('/') {
+      script = format!("{}const {} = require('{}');\n", script, module, module);
+    }
+  }
+
+  js_runtime.execute_script("setup_node_builtins.js", &script)?;
+  Ok(())
+}
