@@ -37,11 +37,13 @@ use super::config::ConfigSnapshot;
 use super::config::SETTINGS_SECTION;
 use super::diagnostics;
 use super::diagnostics::DiagnosticSource;
+use super::diagnostics::DiagnosticsServer;
 use super::documents::to_hover_text;
 use super::documents::to_lsp_range;
 use super::documents::AssetOrDocument;
 use super::documents::Documents;
 use super::documents::LanguageId;
+use super::logging::lsp_log;
 use super::lsp_custom;
 use super::parent_process_checker;
 use super::performance::Performance;
@@ -51,6 +53,7 @@ use super::text;
 use super::tsc;
 use super::tsc::AssetDocument;
 use super::tsc::Assets;
+use super::tsc::AssetsSnapshot;
 use super::tsc::TsServer;
 use super::urls;
 use crate::config_file::ConfigFile;
@@ -61,7 +64,7 @@ use crate::deno_dir;
 use crate::file_fetcher::get_source_from_data_url;
 use crate::fs_util;
 use crate::logger;
-use crate::lsp::logging::lsp_log;
+use crate::proc_state::import_map_from_text;
 use crate::tools::fmt::format_file;
 use crate::tools::fmt::format_parsed_source;
 
@@ -71,17 +74,12 @@ const CACHE_PATH: &str = "deps";
 #[derive(Debug, Clone)]
 pub struct LanguageServer(Arc<tokio::sync::Mutex<Inner>>);
 
+/// Snapshot of the state used by TSC.
 #[derive(Debug, Default)]
 pub(crate) struct StateSnapshot {
-  pub assets: Assets,
-  pub config: ConfigSnapshot,
+  pub assets: AssetsSnapshot,
   pub documents: Documents,
-  pub maybe_lint_config: Option<LintConfig>,
-  pub maybe_fmt_config: Option<FmtConfig>,
-  pub maybe_config_uri: Option<ModuleSpecifier>,
-  pub module_registries: registries::ModuleRegistry,
-  pub performance: Performance,
-  pub url_map: urls::LspUrlMap,
+  pub root_uri: Option<Url>,
 }
 
 #[derive(Debug)]
@@ -96,7 +94,7 @@ pub(crate) struct Inner {
   diagnostics_server: diagnostics::DiagnosticsServer,
   /// The collection of documents that the server is currently handling, either
   /// on disk or "open" within the client.
-  documents: Documents,
+  pub(crate) documents: Documents,
   /// Handles module registries, which allow discovery of modules
   module_registries: registries::ModuleRegistry,
   /// The path to the module registries cache
@@ -110,18 +108,17 @@ pub(crate) struct Inner {
   /// options.
   maybe_config_file: Option<ConfigFile>,
   /// An optional configuration for linter which has been taken from specified config file.
-  maybe_lint_config: Option<LintConfig>,
+  pub(crate) maybe_lint_config: Option<LintConfig>,
   /// An optional configuration for formatter which has been taken from specified config file.
   maybe_fmt_config: Option<FmtConfig>,
-  /// An optional URL which provides the location of a TypeScript configuration
-  /// file which will be used by the Deno LSP.
-  maybe_config_uri: Option<Url>,
   /// An optional import map which is used to resolve modules.
   pub(crate) maybe_import_map: Option<Arc<ImportMap>>,
   /// The URL for the import map which is used to determine relative imports.
   maybe_import_map_uri: Option<Url>,
   /// A collection of measurements which instrument that performance of the LSP.
-  performance: Performance,
+  performance: Arc<Performance>,
+  /// Root provided by the initialization parameters.
+  root_uri: Option<Url>,
   /// A memoized version of fixable diagnostic codes retrieved from TypeScript.
   ts_fixable_diagnostics: Vec<String>,
   /// An abstraction that handles interactions with TypeScript.
@@ -146,26 +143,33 @@ impl Inner {
       registries::ModuleRegistry::new(&module_registries_location);
     let location = dir.root.join(CACHE_PATH);
     let documents = Documents::new(&location);
-    let ts_server = Arc::new(TsServer::new());
+    let performance = Arc::new(Performance::default());
+    let ts_server = Arc::new(TsServer::new(performance.clone()));
     let config = Config::new(client.clone());
+    let diagnostics_server = DiagnosticsServer::new(
+      client.clone(),
+      performance.clone(),
+      ts_server.clone(),
+    );
+    let assets = Assets::new(ts_server.clone());
 
     Self {
-      assets: Default::default(),
+      assets,
       client,
       config,
-      diagnostics_server: Default::default(),
+      diagnostics_server,
       documents,
       maybe_cache_path: None,
       maybe_lint_config: None,
       maybe_fmt_config: None,
       maybe_cache_server: None,
       maybe_config_file: None,
-      maybe_config_uri: None,
       maybe_import_map: None,
       maybe_import_map_uri: None,
       module_registries,
       module_registries_location,
-      performance: Default::default(),
+      root_uri: None,
+      performance,
       ts_fixable_diagnostics: Default::default(),
       ts_server,
       url_map: Default::default(),
@@ -175,7 +179,7 @@ impl Inner {
   /// Searches assets and open documents which might be performed asynchronously,
   /// hydrating in memory caches for subsequent requests.
   pub(crate) async fn get_asset_or_document(
-    &mut self,
+    &self,
     specifier: &ModuleSpecifier,
   ) -> LspResult<AssetOrDocument> {
     self
@@ -195,7 +199,7 @@ impl Inner {
   /// Searches assets and open documents which might be performed asynchronously,
   /// hydrating in memory caches for subsequent requests.
   pub(crate) async fn get_maybe_asset_or_document(
-    &mut self,
+    &self,
     specifier: &ModuleSpecifier,
   ) -> LspResult<Option<AssetOrDocument>> {
     let mark = self.performance.mark(
@@ -203,7 +207,11 @@ impl Inner {
       Some(json!({ "specifier": specifier })),
     );
     let result = if specifier.scheme() == "asset" {
-      self.get_asset(specifier).await?.map(AssetOrDocument::Asset)
+      self
+        .assets
+        .get(specifier, || self.snapshot())
+        .await?
+        .map(AssetOrDocument::Asset)
     } else {
       self.documents.get(specifier).map(AssetOrDocument::Document)
     };
@@ -239,13 +247,9 @@ impl Inner {
     if specifier.scheme() == "asset" {
       self
         .assets
-        .get(specifier)
-        .map(|maybe_asset| {
-          maybe_asset
-            .as_ref()
-            .map(|asset| AssetOrDocument::Asset(asset.clone()))
-        })
+        .get_cached(specifier)
         .flatten()
+        .map(AssetOrDocument::Asset)
     } else {
       self.documents.get(specifier).map(AssetOrDocument::Document)
     }
@@ -267,7 +271,7 @@ impl Inner {
         let navigation_tree: tsc::NavigationTree = self
           .ts_server
           .request(
-            self.snapshot()?,
+            self.snapshot(),
             tsc::RequestMethod::GetNavigationTree(specifier.clone()),
           )
           .await?;
@@ -292,11 +296,9 @@ impl Inner {
 
   /// Returns a tuple with parsed `ConfigFile` and `Url` pointing to that file.
   /// If there's no config file specified in settings returns `None`.
-  fn get_config_file_and_url(
-    &self,
-  ) -> Result<Option<(ConfigFile, Url)>, AnyError> {
+  fn get_config_file(&self) -> Result<Option<ConfigFile>, AnyError> {
     let workspace_settings = self.config.get_workspace_settings();
-    let maybe_root_uri = self.config.root_uri.clone();
+    let maybe_root_uri = self.root_uri.clone();
     let maybe_config = workspace_settings.config;
     if let Some(config_str) = &maybe_config {
       if !config_str.is_empty() {
@@ -316,11 +318,27 @@ impl Inner {
         lsp_log!("  Resolved configuration file: \"{}\"", config_url);
 
         let config_file = ConfigFile::from_specifier(&config_url)?;
-        return Ok(Some((config_file, config_url)));
+        return Ok(Some(config_file));
       }
     }
 
-    Ok(None)
+    // Auto-discover config
+
+    // It is possible that root_uri is not set, for example when having a single
+    // file open and not a workspace.  In those situations we can't
+    // automatically discover the configuration
+    if let Some(root_uri) = maybe_root_uri {
+      let root_path = root_uri.to_file_path().unwrap();
+      let mut checked = std::collections::HashSet::new();
+      let maybe_config =
+        crate::config_file::discover_from(&root_path, &mut checked)?;
+      Ok(maybe_config.map(|c| {
+        lsp_log!("  Auto-resolved configuration file: \"{}\"", c.specifier);
+        c
+      }))
+    } else {
+      Ok(None)
+    }
   }
 
   fn is_diagnosable(&self, specifier: &ModuleSpecifier) -> bool {
@@ -349,7 +367,7 @@ impl Inner {
   }
 
   fn merge_user_tsconfig(
-    &mut self,
+    &self,
     tsconfig: &mut TsConfig,
   ) -> Result<(), AnyError> {
     if let Some(config_file) = self.maybe_config_file.as_ref() {
@@ -365,39 +383,24 @@ impl Inner {
     Ok(())
   }
 
-  pub(crate) fn snapshot(&self) -> LspResult<Arc<StateSnapshot>> {
-    Ok(Arc::new(StateSnapshot {
-      assets: self.assets.clone(),
-      config: self.config.snapshot().map_err(|err| {
-        error!("{}", err);
-        LspError::internal_error()
-      })?,
+  pub(crate) fn snapshot(&self) -> Arc<StateSnapshot> {
+    Arc::new(StateSnapshot {
+      assets: self.assets.snapshot(),
       documents: self.documents.clone(),
-      maybe_lint_config: self.maybe_lint_config.clone(),
-      maybe_fmt_config: self.maybe_fmt_config.clone(),
-      maybe_config_uri: self.maybe_config_uri.clone(),
-      module_registries: self.module_registries.clone(),
-      performance: self.performance.clone(),
-      url_map: self.url_map.clone(),
-    }))
+      root_uri: self.root_uri.clone(),
+    })
   }
 
   pub fn update_cache(&mut self) -> Result<(), AnyError> {
     let mark = self.performance.mark("update_cache", None::<()>);
     self.performance.measure(mark);
     self.maybe_cache_server = None;
-    let (maybe_cache, maybe_root_uri) = {
-      let config = &self.config;
-      (
-        config.get_workspace_settings().cache,
-        config.root_uri.clone(),
-      )
-    };
+    let maybe_cache = self.config.get_workspace_settings().cache;
     let maybe_cache_path = if let Some(cache_str) = &maybe_cache {
       lsp_log!("Setting cache path from: \"{}\"", cache_str);
       let cache_url = if let Ok(url) = Url::from_file_path(cache_str) {
         Ok(url)
-      } else if let Some(root_uri) = &maybe_root_uri {
+      } else if let Some(root_uri) = &self.root_uri {
         let root_path = fs_util::specifier_to_file_path(root_uri)?;
         let cache_path = root_path.join(cache_str);
         Url::from_file_path(cache_path).map_err(|_| {
@@ -437,13 +440,7 @@ impl Inner {
   pub async fn update_import_map(&mut self) -> Result<(), AnyError> {
     let mark = self.performance.mark("update_import_map", None::<()>);
     self.maybe_cache_server = None;
-    let (maybe_import_map, maybe_root_uri) = {
-      let config = &self.config;
-      (
-        config.get_workspace_settings().import_map,
-        config.root_uri.clone(),
-      )
-    };
+    let maybe_import_map = self.config.get_workspace_settings().import_map;
     if let Some(import_map_str) = &maybe_import_map {
       lsp_log!("Setting import map from: \"{}\"", import_map_str);
       let import_map_url = if let Ok(url) = Url::from_file_path(import_map_str)
@@ -453,7 +450,7 @@ impl Inner {
         Url::parse(import_map_str).map_err(|_| {
           anyhow!("Bad data url for import map: {:?}", import_map_str)
         })
-      } else if let Some(root_uri) = &maybe_root_uri {
+      } else if let Some(root_uri) = &self.root_uri {
         let root_path = fs_util::specifier_to_file_path(root_uri)?;
         let import_map_path = root_path.join(import_map_str);
         Url::from_file_path(import_map_path).map_err(|_| {
@@ -482,12 +479,9 @@ impl Inner {
           )
         })?
       };
-      let import_map = Arc::new(ImportMap::from_json(
-        &import_map_url.to_string(),
-        &import_map_json,
-      )?);
+      let import_map = import_map_from_text(&import_map_url, &import_map_json)?;
       self.maybe_import_map_uri = Some(import_map_url);
-      self.maybe_import_map = Some(import_map);
+      self.maybe_import_map = Some(Arc::new(import_map));
     } else {
       self.maybe_import_map = None;
     }
@@ -523,13 +517,10 @@ impl Inner {
 
   fn update_config_file(&mut self) -> Result<(), AnyError> {
     self.maybe_config_file = None;
-    self.maybe_config_uri = None;
     self.maybe_fmt_config = None;
     self.maybe_lint_config = None;
 
-    let maybe_file_and_url = self.get_config_file_and_url()?;
-
-    if let Some((config_file, config_url)) = maybe_file_and_url {
+    if let Some(config_file) = self.get_config_file()? {
       let lint_config = config_file
         .to_lint_config()
         .map_err(|err| {
@@ -544,7 +535,6 @@ impl Inner {
         .unwrap_or_default();
 
       self.maybe_config_file = Some(config_file);
-      self.maybe_config_uri = Some(config_url);
       self.maybe_lint_config = Some(lint_config);
       self.maybe_fmt_config = Some(fmt_config);
     }
@@ -583,30 +573,11 @@ impl Inner {
     }
     let _ok: bool = self
       .ts_server
-      .request(self.snapshot()?, tsc::RequestMethod::Configure(tsconfig))
+      .request(self.snapshot(), tsc::RequestMethod::Configure(tsconfig))
       .await?;
     self.performance.measure(mark);
     Ok(())
   }
-
-  async fn get_asset(
-    &mut self,
-    specifier: &ModuleSpecifier,
-  ) -> LspResult<Option<AssetDocument>> {
-    if let Some(maybe_asset) = self.assets.get(specifier) {
-      Ok(maybe_asset.clone())
-    } else {
-      let maybe_asset =
-        tsc::get_asset(specifier, &self.ts_server, self.snapshot()?)
-          .await
-          .map_err(|err| {
-            error!("Error getting asset {}: {}", specifier, err);
-            LspError::internal_error()
-          })?;
-      self.assets.insert(specifier.clone(), maybe_asset.clone());
-      Ok(maybe_asset)
-    }
-  }
 }
 
 // lspower::LanguageServer methods. This file's LanguageServer delegates to us.
@@ -650,20 +621,19 @@ impl Inner {
     }
 
     {
-      let config = &mut self.config;
       // sometimes this root uri may not have a trailing slash, so force it to
-      config.root_uri = params
+      self.root_uri = params
         .root_uri
         .map(|s| self.url_map.normalize_url(&s))
         .map(fs_util::ensure_directory_specifier);
 
       if let Some(value) = params.initialization_options {
-        config.set_workspace_settings(value).map_err(|err| {
+        self.config.set_workspace_settings(value).map_err(|err| {
           error!("Cannot set workspace settings: {}", err);
           LspError::internal_error()
         })?;
       }
-      config.update_capabilities(&params.capabilities);
+      self.config.update_capabilities(&params.capabilities);
     }
 
     self.update_debug_flag();
@@ -681,7 +651,7 @@ impl Inner {
     if capabilities.code_action_provider.is_some() {
       let fixable_diagnostics: Vec<String> = self
         .ts_server
-        .request(self.snapshot()?, tsc::RequestMethod::GetSupportedCodeFixes)
+        .request(self.snapshot(), tsc::RequestMethod::GetSupportedCodeFixes)
         .await
         .map_err(|err| {
           error!("Unable to get fixable diagnostics: {}", err);
@@ -942,8 +912,8 @@ impl Inner {
       }
     }
     // if the current tsconfig has changed, we need to reload it
-    if let Some(config_uri) = &self.maybe_config_uri {
-      if changes.iter().any(|uri| config_uri == uri) {
+    if let Some(config_file) = &self.maybe_config_file {
+      if changes.iter().any(|uri| config_file.specifier == *uri) {
         if let Err(err) = self.update_config_file() {
           self.client.show_message(MessageType::WARNING, err).await;
         }
@@ -1071,7 +1041,7 @@ impl Inner {
     }
   }
 
-  async fn hover(&mut self, params: HoverParams) -> LspResult<Option<Hover>> {
+  async fn hover(&self, params: HoverParams) -> LspResult<Option<Hover>> {
     let specifier = self
       .url_map
       .normalize_url(&params.text_document_position_params.text_document.uri);
@@ -1119,6 +1089,12 @@ impl Inner {
         ),
         (None, None, _) => unreachable!("{}", json!(params)),
       };
+      let value =
+        if let Some(docs) = self.module_registries.get_hover(&dep).await {
+          format!("{}\n\n---\n\n{}", value, docs)
+        } else {
+          value
+        };
       Some(Hover {
         contents: HoverContents::Markup(MarkupContent {
           kind: MarkupKind::Markdown,
@@ -1134,7 +1110,7 @@ impl Inner {
       ));
       let maybe_quick_info: Option<tsc::QuickInfo> = self
         .ts_server
-        .request(self.snapshot()?, req)
+        .request(self.snapshot(), req)
         .await
         .map_err(|err| {
           error!("Unable to get quick info: {}", err);
@@ -1147,7 +1123,7 @@ impl Inner {
   }
 
   async fn code_action(
-    &mut self,
+    &self,
     params: CodeActionParams,
   ) -> LspResult<Option<CodeActionResponse>> {
     let specifier = self.url_map.normalize_url(&params.text_document.uri);
@@ -1214,7 +1190,7 @@ impl Inner {
               codes,
             ));
             let actions: Vec<tsc::CodeFixAction> =
-              match self.ts_server.request(self.snapshot()?, req).await {
+              match self.ts_server.request(self.snapshot(), req).await {
                 Ok(items) => items,
                 Err(err) => {
                   // sometimes tsc reports errors when retrieving code actions
@@ -1287,7 +1263,7 @@ impl Inner {
     ));
     let refactor_infos: Vec<tsc::ApplicableRefactorInfo> = self
       .ts_server
-      .request(self.snapshot()?, req)
+      .request(self.snapshot(), req)
       .await
       .map_err(|err| {
         error!("Failed to request to tsserver {}", err);
@@ -1321,7 +1297,7 @@ impl Inner {
   }
 
   async fn code_action_resolve(
-    &mut self,
+    &self,
     params: CodeAction,
   ) -> LspResult<CodeAction> {
     if params.kind.is_none() || params.data.is_none() {
@@ -1334,6 +1310,7 @@ impl Inner {
 
     let result = if kind.as_str().starts_with(CodeActionKind::QUICKFIX.as_str())
     {
+      let snapshot = self.snapshot();
       let code_action_data: CodeActionData =
         from_value(data).map_err(|err| {
           error!("Unable to decode code action data: {}", err);
@@ -1345,7 +1322,7 @@ impl Inner {
       ));
       let combined_code_actions: tsc::CombinedCodeActions = self
         .ts_server
-        .request(self.snapshot()?, req)
+        .request(snapshot.clone(), req)
         .await
         .map_err(|err| {
           error!("Unable to get combined fix from TypeScript: {}", err);
@@ -1360,7 +1337,7 @@ impl Inner {
         fix_ts_import_changes(
           &code_action_data.specifier,
           &combined_code_actions.changes,
-          self,
+          &self.documents,
         )
         .map_err(|err| {
           error!("Unable to remap changes: {}", err);
@@ -1377,6 +1354,7 @@ impl Inner {
         })?;
       code_action
     } else if kind.as_str().starts_with(CodeActionKind::REFACTOR.as_str()) {
+      let snapshot = self.snapshot();
       let mut code_action = params.clone();
       let action_data: refactor::RefactorCodeActionData = from_value(data)
         .map_err(|err| {
@@ -1394,11 +1372,8 @@ impl Inner {
         action_data.refactor_name.clone(),
         action_data.action_name.clone(),
       ));
-      let refactor_edit_info: tsc::RefactorEditInfo = self
-        .ts_server
-        .request(self.snapshot()?, req)
-        .await
-        .map_err(|err| {
+      let refactor_edit_info: tsc::RefactorEditInfo =
+        self.ts_server.request(snapshot, req).await.map_err(|err| {
           error!("Failed to request to tsserver {}", err);
           LspError::invalid_request()
         })?;
@@ -1460,7 +1435,7 @@ impl Inner {
   }
 
   async fn code_lens_resolve(
-    &mut self,
+    &self,
     code_lens: CodeLens,
   ) -> LspResult<CodeLens> {
     let mark = self.performance.mark("code_lens_resolve", Some(&code_lens));
@@ -1481,7 +1456,7 @@ impl Inner {
   }
 
   async fn document_highlight(
-    &mut self,
+    &self,
     params: DocumentHighlightParams,
   ) -> LspResult<Option<Vec<DocumentHighlight>>> {
     let specifier = self
@@ -1504,7 +1479,7 @@ impl Inner {
     ));
     let maybe_document_highlights: Option<Vec<tsc::DocumentHighlights>> = self
       .ts_server
-      .request(self.snapshot()?, req)
+      .request(self.snapshot(), req)
       .await
       .map_err(|err| {
         error!("Unable to get document highlights from TypeScript: {}", err);
@@ -1526,7 +1501,7 @@ impl Inner {
   }
 
   async fn references(
-    &mut self,
+    &self,
     params: ReferenceParams,
   ) -> LspResult<Option<Vec<Location>>> {
     let specifier = self
@@ -1547,7 +1522,7 @@ impl Inner {
     ));
     let maybe_references: Option<Vec<tsc::ReferenceEntry>> = self
       .ts_server
-      .request(self.snapshot()?, req)
+      .request(self.snapshot(), req)
       .await
       .map_err(|err| {
         error!("Unable to get references from TypeScript: {}", err);
@@ -1569,7 +1544,8 @@ impl Inner {
             self.get_asset_or_document(&reference_specifier).await?;
           asset_or_doc.line_index()
         };
-        results.push(reference.to_location(reference_line_index, self));
+        results
+          .push(reference.to_location(reference_line_index, &self.url_map));
       }
 
       self.performance.measure(mark);
@@ -1581,7 +1557,7 @@ impl Inner {
   }
 
   async fn goto_definition(
-    &mut self,
+    &self,
     params: GotoDefinitionParams,
   ) -> LspResult<Option<GotoDefinitionResponse>> {
     let specifier = self
@@ -1602,7 +1578,7 @@ impl Inner {
     ));
     let maybe_definition: Option<tsc::DefinitionInfoAndBoundSpan> = self
       .ts_server
-      .request(self.snapshot()?, req)
+      .request(self.snapshot(), req)
       .await
       .map_err(|err| {
         error!("Unable to get definition from TypeScript: {}", err);
@@ -1620,7 +1596,7 @@ impl Inner {
   }
 
   async fn goto_type_definition(
-    &mut self,
+    &self,
     params: GotoTypeDefinitionParams,
   ) -> LspResult<Option<GotoTypeDefinitionResponse>> {
     let specifier = self
@@ -1642,7 +1618,7 @@ impl Inner {
     };
     let maybe_definition_info: Option<Vec<tsc::DefinitionInfo>> = self
       .ts_server
-      .request(self.snapshot()?, req)
+      .request(self.snapshot(), req)
       .await
       .map_err(|err| {
         error!("Unable to get type definition from TypeScript: {}", err);
@@ -1668,7 +1644,7 @@ impl Inner {
   }
 
   async fn completion(
-    &mut self,
+    &self,
     params: CompletionParams,
   ) -> LspResult<Option<CompletionResponse>> {
     let specifier = self
@@ -1686,12 +1662,13 @@ impl Inner {
     // completions, we will use internal logic and if there are completions
     // for imports, we will return those and not send a message into tsc, where
     // other completions come from.
-    let snapshot = self.snapshot()?;
     let response = if let Some(response) = completions::get_import_completions(
       &specifier,
       &params.text_document_position.position,
-      &snapshot,
+      &self.config.snapshot(),
       self.client.clone(),
+      &self.module_registries,
+      &self.documents,
     )
     .await
     {
@@ -1720,11 +1697,9 @@ impl Inner {
           trigger_character,
         },
       ));
-      let maybe_completion_info: Option<tsc::CompletionInfo> = self
-        .ts_server
-        .request(self.snapshot()?, req)
-        .await
-        .map_err(|err| {
+      let snapshot = self.snapshot();
+      let maybe_completion_info: Option<tsc::CompletionInfo> =
+        self.ts_server.request(snapshot, req).await.map_err(|err| {
           error!("Unable to get completion info from TypeScript: {}", err);
           LspError::internal_error()
         })?;
@@ -1746,7 +1721,7 @@ impl Inner {
   }
 
   async fn completion_resolve(
-    &mut self,
+    &self,
     params: CompletionItem,
   ) -> LspResult<CompletionItem> {
     let mark = self.performance.mark("completion_resolve", Some(&params));
@@ -1760,14 +1735,13 @@ impl Inner {
         })?;
       if let Some(data) = data.tsc {
         let req = tsc::RequestMethod::GetCompletionDetails(data.into());
-        let maybe_completion_info: Option<tsc::CompletionEntryDetails> = self
-          .ts_server
-          .request(self.snapshot()?, req)
-          .await
-          .map_err(|err| {
-            error!("Unable to get completion info from TypeScript: {}", err);
-            LspError::internal_error()
-          })?;
+        let maybe_completion_info: Option<tsc::CompletionEntryDetails> =
+          self.ts_server.request(self.snapshot(), req).await.map_err(
+            |err| {
+              error!("Unable to get completion info from TypeScript: {}", err);
+              LspError::internal_error()
+            },
+          )?;
         if let Some(completion_info) = maybe_completion_info {
           completion_info.as_completion_item(&params)
         } else {
@@ -1793,7 +1767,7 @@ impl Inner {
   }
 
   async fn goto_implementation(
-    &mut self,
+    &self,
     params: GotoImplementationParams,
   ) -> LspResult<Option<GotoImplementationResponse>> {
     let specifier = self
@@ -1815,7 +1789,7 @@ impl Inner {
     ));
     let maybe_implementations: Option<Vec<tsc::ImplementationLocation>> = self
       .ts_server
-      .request(self.snapshot()?, req)
+      .request(self.snapshot(), req)
       .await
       .map_err(|err| {
         error!("Failed to request to tsserver {}", err);
@@ -1841,7 +1815,7 @@ impl Inner {
   }
 
   async fn folding_range(
-    &mut self,
+    &self,
     params: FoldingRangeParams,
   ) -> LspResult<Option<Vec<FoldingRange>>> {
     let specifier = self.url_map.normalize_url(&params.text_document.uri);
@@ -1857,7 +1831,7 @@ impl Inner {
     let req = tsc::RequestMethod::GetOutliningSpans(specifier.clone());
     let outlining_spans: Vec<tsc::OutliningSpan> = self
       .ts_server
-      .request(self.snapshot()?, req)
+      .request(self.snapshot(), req)
       .await
       .map_err(|err| {
         error!("Failed to request to tsserver {}", err);
@@ -1885,7 +1859,7 @@ impl Inner {
   }
 
   async fn incoming_calls(
-    &mut self,
+    &self,
     params: CallHierarchyIncomingCallsParams,
   ) -> LspResult<Option<Vec<CallHierarchyIncomingCall>>> {
     let specifier = self.url_map.normalize_url(&params.item.uri);
@@ -1905,7 +1879,7 @@ impl Inner {
     ));
     let incoming_calls: Vec<tsc::CallHierarchyIncomingCall> = self
       .ts_server
-      .request(self.snapshot()?, req)
+      .request(self.snapshot(), req)
       .await
       .map_err(|err| {
         error!("Failed to request to tsserver {}", err);
@@ -1913,7 +1887,6 @@ impl Inner {
       })?;
 
     let maybe_root_path_owned = self
-      .config
       .root_uri
       .as_ref()
       .and_then(|uri| fs_util::specifier_to_file_path(uri).ok());
@@ -1934,7 +1907,7 @@ impl Inner {
   }
 
   async fn outgoing_calls(
-    &mut self,
+    &self,
     params: CallHierarchyOutgoingCallsParams,
   ) -> LspResult<Option<Vec<CallHierarchyOutgoingCall>>> {
     let specifier = self.url_map.normalize_url(&params.item.uri);
@@ -1954,7 +1927,7 @@ impl Inner {
     ));
     let outgoing_calls: Vec<tsc::CallHierarchyOutgoingCall> = self
       .ts_server
-      .request(self.snapshot()?, req)
+      .request(self.snapshot(), req)
       .await
       .map_err(|err| {
         error!("Failed to request to tsserver {}", err);
@@ -1962,7 +1935,6 @@ impl Inner {
       })?;
 
     let maybe_root_path_owned = self
-      .config
       .root_uri
       .as_ref()
       .and_then(|uri| fs_util::specifier_to_file_path(uri).ok());
@@ -1984,7 +1956,7 @@ impl Inner {
   }
 
   async fn prepare_call_hierarchy(
-    &mut self,
+    &self,
     params: CallHierarchyPrepareParams,
   ) -> LspResult<Option<Vec<CallHierarchyItem>>> {
     let specifier = self
@@ -2009,7 +1981,7 @@ impl Inner {
     let maybe_one_or_many: Option<tsc::OneOrMany<tsc::CallHierarchyItem>> =
       self
         .ts_server
-        .request(self.snapshot()?, req)
+        .request(self.snapshot(), req)
         .await
         .map_err(|err| {
           error!("Failed to request to tsserver {}", err);
@@ -2018,7 +1990,6 @@ impl Inner {
 
     let response = if let Some(one_or_many) = maybe_one_or_many {
       let maybe_root_path_owned = self
-        .config
         .root_uri
         .as_ref()
         .and_then(|uri| fs_util::specifier_to_file_path(uri).ok());
@@ -2058,7 +2029,7 @@ impl Inner {
   }
 
   async fn rename(
-    &mut self,
+    &self,
     params: RenameParams,
   ) -> LspResult<Option<WorkspaceEdit>> {
     let specifier = self
@@ -2085,7 +2056,7 @@ impl Inner {
 
     let maybe_locations: Option<Vec<tsc::RenameLocation>> = self
       .ts_server
-      .request(self.snapshot()?, req)
+      .request(self.snapshot(), req)
       .await
       .map_err(|err| {
         error!("Failed to request to tsserver {}", err);
@@ -2171,7 +2142,7 @@ impl Inner {
 
       let selection_range: tsc::SelectionRange = self
         .ts_server
-        .request(self.snapshot()?, req)
+        .request(self.snapshot(), req)
         .await
         .map_err(|err| {
           error!("Failed to request to tsserver {}", err);
@@ -2209,7 +2180,7 @@ impl Inner {
     ));
     let semantic_classification: tsc::Classifications = self
       .ts_server
-      .request(self.snapshot()?, req)
+      .request(self.snapshot(), req)
       .await
       .map_err(|err| {
         error!("Failed to request to tsserver {}", err);
@@ -2252,7 +2223,7 @@ impl Inner {
     ));
     let semantic_classification: tsc::Classifications = self
       .ts_server
-      .request(self.snapshot()?, req)
+      .request(self.snapshot(), req)
       .await
       .map_err(|err| {
         error!("Failed to request to tsserver {}", err);
@@ -2305,7 +2276,7 @@ impl Inner {
     ));
     let maybe_signature_help_items: Option<tsc::SignatureHelpItems> = self
       .ts_server
-      .request(self.snapshot()?, req)
+      .request(self.snapshot(), req)
       .await
       .map_err(|err| {
         error!("Failed to request to tsserver: {}", err);
@@ -2337,7 +2308,7 @@ impl Inner {
 
     let navigate_to_items: Vec<tsc::NavigateToItem> = self
       .ts_server
-      .request(self.snapshot()?, req)
+      .request(self.snapshot(), req)
       .await
       .map_err(|err| {
         error!("Failed request to tsserver: {}", err);
@@ -2368,11 +2339,7 @@ impl lspower::LanguageServer for LanguageServer {
     params: InitializeParams,
   ) -> LspResult<InitializeResult> {
     let mut language_server = self.0.lock().await;
-    let client = language_server.client.clone();
-    let ts_server = language_server.ts_server.clone();
-    language_server
-      .diagnostics_server
-      .start(self.0.clone(), client, ts_server);
+    language_server.diagnostics_server.start(self.0.clone());
     language_server.initialize(params).await
   }
 
